var Revolver = {

  // Member variables
  cmdStack: new Array(),
  cmdInd: 0,
  outputbuffer: 10000000,

  keyPress: function (e, scWin) {
    if (e.keyCode == 13) {
      if (e.shiftKey) //allows you to create a breakline
        return true;

      var input = $('input');
      var output = $('output');
      if (input != null && output != null) {
        var command = input.value;

        if (command == "clear") {
          input.value = "";
          output.innerHTML = "";
          return false;
        }

        Revolver.cmdStack.push(command);
        Revolver.execute(command);
        Revolver.cmdInd = 0;
        return false;
      }
      else
        alert("Error! Failed to find control in function Revolver.keyPress()");
    }

    if (e.keyCode == 38) { //presses the up arrow to scroll command cache
      Revolver.cmdInd++;
      if (Revolver.cmdInd >= Revolver.cmdStack.length)
        Revolver.cmdInd = Revolver.cmdStack.length;
      Revolver.useCachedCommand();

      return false;
    }

    if (e.keyCode == 40) { //presses the down arrow to scroll command cache
      Revolver.cmdInd--;
      if (Revolver.cmdInd < 0)
        Revolver.cmdInd = 0;
      Revolver.useCachedCommand();

      return false;
    }

    return true;
  },

  useCachedCommand: function () {
    var input = $('input');
    if (input != null) {
      if (Revolver.cmdInd == 0)
        input.value = "";
      else
        input.value = Revolver.cmdStack[Revolver.cmdStack.length - Revolver.cmdInd];
    }
    else
      alert("Error! Failed to find control in function Revolver.useCachedCommand()");
  },

  execute: function (command) {
    var status = $('status');
    status.innerHTML = 'communicating with server';

    var input = $('input');
    var request = new scRequest();
    input.disabled = true;

    var prepCommand = encodeURIComponent(command);

    // ensure the parenthesis are also encoded to the Sitecore command handler doesn't choke when trying to parse the command
    prepCommand = prepCommand.replace('(', '%28').replace(')', '%29');

    var parameters = 'sessionId=' + $('hdnSessionId').value + '&command=' + prepCommand;

    if ($('__CSRFTOKEN'))
      request.form = '__CSRFTOKEN=' + $('__CSRFTOKEN').value;

    request.build("", "", "", "r:" + parameters, true, Revolver.contextmenu, Revolver.modified);

    request.callback = function (response) {
      status.innerHTML = 'awaiting input';
      var output = $('output');

      if (input != null && output != null) {
        var result = {};

        if (typeof (JSON) != 'undefined')
          result = JSON.parse(response);
        else
          result = response.evalJSON(true);

        if (result.exit != null && result.exit) {
          parent.scWin.closeWindow();
          return;
        }

        if (result.prompt != null) {
          $('prompt').innerHTML = result.prompt;

          var promptOutput = document.createElement('div');
          promptOutput.innerHTML = result.prompt;
          output.appendChild(promptOutput);
        }

        // todo, limit the output buffer size
        if (result.outputBuffer != null)
          Revolver.outputbuffer = parseInt(result.outputBuffer);

        var commandOutput = document.createElement('div');
        commandOutput.innerHTML = command;
        output.appendChild(commandOutput);

        var outel = document.createElement('div');
        Element.extend(outel);

        if (result.fatal) {
          outel.addClassName("error");
          outel.innerHTML = '<br/>' + result.fatal.escapeHTML();
          input.keyPress = null;
        }

        if (result.error) {
          outel.addClassName("error");
          outel.innerHTML = result.error.escapeHTML();
        } else {
          outel.addClassName("output");
          outel.appendChild(document.createElement('br'));

          if (result.output)
              outel.innerHTML = '<pre>' + result.output.escapeHTML() + '</pre>';

          if (result.outputLines)
            for (var line in result.outputLines) {
              var inner = document.createElement('pre');
              inner.innerHTML = line.escapeHTML();
              outel.appendChild(inner);
            }
        }

        output.appendChild(outel);
        outel.scrollIntoView(false);
        input.value = "";
        input.disabled = false;
        Revolver.checkInputDimension();
        input.focus();
      }
    };

    request.async = true;
    request.execute();
  },

  checkInputDimension: function () {
    var input = $('input');
    if (input != null) {
      if (input.scrollHeight != input.clientHeight)
        input.style.height = input.scrollHeight + "px";
    }
  },

  setupCursorFocus: function () {
    $('scrollbox').observe('click', Revolver.inputFocus);
    $('prompt').observe('click', Revolver.inputFocus);
    $('input').observe('click', Revolver.inputFocus);
    $('statusBorder').observe('click', Revolver.inputFocus);
  },

  inputFocus: function () {
    $('input').focus();
  }
};

document.observe("dom:loaded", function () {
  Revolver.checkInputDimension();
  Revolver.setupCursorFocus();

  // Fix percentage height issue for output box in FF
  if (navigator.userAgent.indexOf("Gecko") > -1) {
    var output = $('output');
    if (output != null)
      output.style.height = "200px";
  }

  var input = $('input');
  if (input != null)
    input.focus();
});

scIEOnSelectStart = function () {
  return;
}